@@ -1,6 +1,6 @@
 /*
     fileio
-    copyright (c) 1998-2011 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2012 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -60,230 +60,137 @@ fileio_get_absolute_path (const gchar *file,
   gint i = 0, j, s;
   DWORD dwSize;             /* ja:フルパスの長さ */
   HANDLE hFind;             /* ja:検索ハンドル */
-# if ! GLIB_CHECK_VERSION(2,6,0)
   LPTSTR p, lpszFile;
   TCHAR szFile[MAX_PATH];   /* ja:フルパス */
   TCHAR szLongName[MAX_PATH];
-# endif /* not GLIB_CHECK_VERSION(2,6,0) */
 
   if (!file)
     return NULL;
-# if GLIB_CHECK_VERSION(2,6,0)
-  if (G_WIN32_HAVE_WIDECHAR_API ())
+# ifdef UNICODE
+  utf8str = g_filename_to_utf8 (file, -1, NULL, NULL, NULL);
+  if (!utf8str)
+    return g_strdup (file);
+  lpszFile = g_utf8_to_utf16 (utf8str, -1, NULL, NULL, NULL);
+  g_free (utf8str);
+# else /* not UNICODE */
+  lpszFile = g_win32_locale_filename_from_utf8 (file);
+# endif /* not UNICODE */
+  if (PathIsRelative (lpszFile))
     {
-      LPWSTR p, lpszFile;
-      WCHAR szFile[MAX_PATH];   /* ja:フルパス */
-      WCHAR szLongName[MAX_PATH];
-# endif /* GLIB_CHECK_VERSION(2,6,0) */
+      gint leng;
+      LPSTR lpszPath = NULL, lpszTemp;
 
-# if GLIB_CHECK_VERSION(2,6,0) || defined (UNICODE)
-      utf8str = g_filename_to_utf8 (file, -1, NULL, NULL, NULL);
-      if (!utf8str)
-        return g_strdup (file);
-      lpszFile = g_utf8_to_utf16 (utf8str, -1, NULL, NULL, NULL);
-      g_free (utf8str);
-      if (PathIsRelativeW (lpszFile))
+      if (path)
         {
-          gint leng;
-          LPWSTR lpszPath = NULL, lpszTemp;
-
-          if (path)
+# ifdef UNICODE
+          utf8str = g_filename_to_utf8 (path, -1, NULL, NULL, NULL);
+          if (utf8str)
             {
-              utf8str = g_filename_to_utf8 (path, -1, NULL, NULL, NULL);
-              if (utf8str)
-                {
-                  lpszPath = g_utf8_to_utf16 (utf8str, -1, NULL, NULL, NULL);
-                  g_free (utf8str);
-                }
+              lpszPath = g_utf8_to_utf16 (utf8str, -1, NULL, NULL, NULL);
+              g_free (utf8str);
             }
-          if (!lpszPath || PathIsRelativeW (lpszPath))
-            {
-              g_free (lpszPath);
-              lpszPath = g_malloc (MAX_PATH * sizeof (WCHAR));
-              GetCurrentDirectoryW (MAX_PATH, lpszPath);
-            }
-          leng = lstrlenW (lpszPath);
-          if (leng > 0 && G_IS_DIR_SEPARATOR (lpszPath[leng - 1]))
-            lpszPath[--leng] = '\0';
-          lpszTemp = g_malloc ((leng + lstrlenW (lpszFile) + 2)
-                                                            * sizeof (WCHAR));
-          lstrcpyW (lpszTemp, lpszPath);
-          lpszTemp[leng] = G_DIR_SEPARATOR;
-          lpszTemp[leng + 1] = '\0';
-          lstrcatW (lpszTemp, lpszFile);
-          g_free (lpszPath);
-          g_free (lpszFile);
-          lpszFile = lpszTemp;
+# else /* not UNICODE */
+          lpszPath = g_win32_locale_filename_from_utf8 (path);
+# endif /* not UNICODE */
         }
-      dwSize = GetFullPathNameW (lpszFile, MAX_PATH, szFile, &p);
-      g_free (lpszFile);
-      if (dwSize == 0 || dwSize >= MAX_PATH)
-        return g_strdup (file);
-      if (dwSize >= 2)
+      if (!lpszPath || PathIsRelative (lpszPath))
         {
-          if (szFile[1] == ':')
+          g_free (lpszPath);
+          lpszPath = g_malloc (MAX_PATH * sizeof (TCHAR));
+          GetCurrentDirectory (MAX_PATH, lpszPath);
+        }
+      leng = lstrlen (lpszPath);
+      if (leng > 0 && G_IS_DIR_SEPARATOR (lpszPath[leng - 1]))
+        lpszPath[--leng] = '\0';
+      lpszTemp = g_malloc ((leng + lstrlen (lpszFile) + 2) * sizeof (TCHAR));
+      lstrcpy (lpszTemp, lpszPath);
+      lpszTemp[leng] = G_DIR_SEPARATOR;
+      lpszTemp[leng + 1] = '\0';
+      lstrcat (lpszTemp, lpszFile);
+      g_free (lpszPath);
+      g_free (lpszFile);
+      lpszFile = lpszTemp;
+    }
+  dwSize = GetFullPathName (lpszFile, MAX_PATH, szFile, &p);
+  g_free (lpszFile);
+  if (dwSize == 0 || dwSize >= MAX_PATH)
+    return g_strdup (file);
+  if (dwSize >= 2)
+    {
+      if (szFile[1] == ':')
+        {
+          szLongName[0] = (TCHAR)CharUpper ((LPTSTR)szFile[0]);
+          szLongName[1] = ':';
+          if (G_IS_DIR_SEPARATOR (szFile[2]))
             {
-              szLongName[0] = (WCHAR)CharUpperW ((LPWSTR)szFile[0]);
-              szLongName[1] = ':';
-              if (G_IS_DIR_SEPARATOR (szFile[2]))
-                {
-                  szLongName[2] = G_DIR_SEPARATOR;
-                  i = 3;
-                }
-              else
-                {
-                  i = 2;
-                }
+              szLongName[2] = G_DIR_SEPARATOR;
+              i = 3;
             }
-          else if (G_IS_DIR_SEPARATOR (szFile[0])
-                                            && G_IS_DIR_SEPARATOR (szFile[1]))
+          else
             {
               i = 2;
-              while (i < dwSize)
-                {
-                  if (G_IS_DIR_SEPARATOR (szFile[i]))
-                    break;
-                  i++;
-                }
-              i++;
-              g_memmove (szLongName, szFile, i * sizeof (WCHAR));
             }
         }
-      szLongName[i] = '\0';
-      j = i;
-      while (i <= dwSize)
+      else if (G_IS_DIR_SEPARATOR (szFile[0])
+                                            && G_IS_DIR_SEPARATOR (szFile[1]))
         {
-          if (szFile[i] == '\0' || G_IS_DIR_SEPARATOR (szFile[i]))
+          i = 2;
+          while (i < dwSize)
             {
-              WIN32_FIND_DATAW rcf;
-
-              s = lstrlenW (szLongName);
-              g_memmove (szLongName + s, szFile + j, (i - j) * sizeof (WCHAR));
-              szLongName[s + i - j] = '\0';
-              hFind = FindFirstFileW (szLongName, &rcf);
-              if (hFind != INVALID_HANDLE_VALUE)
-                FindClose (hFind);
-              else
-                lstrcpyW (rcf.cFileName, L".");
-              if (lstrcmpW (rcf.cFileName, L".") != 0)
-                {
-                  g_memmove (szLongName + s, rcf.cFileName,
-                                    lstrlenW (rcf.cFileName) * sizeof (WCHAR));
-                  s += lstrlenW (rcf.cFileName);
-                }
-              else
-                {
-                  s += i - j;
-                }
               if (G_IS_DIR_SEPARATOR (szFile[i]))
-                szLongName[s++] = G_DIR_SEPARATOR;
-              szLongName[s] = '\0';
-              j = i + 1;
+                break;
+# ifdef UNICODE
+              i++;
+# else /* not UNICODE */
+              i += IsDBCSLeadByteEx (CP_ACP, szFile[i]) ? 2 : 1;
+# endif /* not UNICODE */
             }
           i++;
+          g_memmove (szLongName, szFile, i * sizeof (TCHAR));
         }
-      utf8str = g_utf16_to_utf8 (szLongName, -1, NULL, NULL, NULL);
-# endif /* GLIB_CHECK_VERSION(2,6,0) || defined (UNICODE) */
-# if GLIB_CHECK_VERSION(2,6,0)
     }
-  else
+  szLongName[i] = '\0';
+  j = i;
+  while (i <= dwSize)
     {
-      LPSTR p, lpszFile;
-      CHAR szFile[MAX_PATH];   /* ja:フルパス */
-      CHAR szLongName[MAX_PATH];
-# endif /* GLIB_CHECK_VERSION(2,6,0) */
-# if GLIB_CHECK_VERSION(2,6,0) || ! defined (UNICODE)
-
-      lpszFile = g_win32_locale_filename_from_utf8 (file);
-      if (PathIsRelativeA (lpszFile))
+      if (szFile[i] == '\0' || G_IS_DIR_SEPARATOR (szFile[i]))
         {
-          LPSTR lpszPath = NULL, lpszTemp;
+          WIN32_FIND_DATA rcf;
 
-          if (path)
-            lpszPath = g_win32_locale_filename_from_utf8 (path);
-          if (!lpszPath || PathIsRelativeA (lpszPath))
+          s = lstrlen (szLongName);
+          g_memmove (szLongName + s, szFile + j, (i - j) * sizeof (TCHAR));
+          szLongName[s + i - j] = '\0';
+          hFind = FindFirstFile (szLongName, &rcf);
+          if (hFind != INVALID_HANDLE_VALUE)
+            FindClose (hFind);
+          else
+            lstrcpy (rcf.cFileName, _T("."));
+          if (lstrcmp (rcf.cFileName, _T(".")) != 0)
             {
-              g_free (lpszPath);
-              lpszPath = g_malloc (MAX_PATH * sizeof (CHAR));
-              GetCurrentDirectoryA (MAX_PATH, lpszPath);
+              g_memmove (szLongName + s, rcf.cFileName,
+                                    lstrlen (rcf.cFileName) * sizeof (TCHAR));
+              s += lstrlen (rcf.cFileName);
             }
-          lpszTemp = g_build_filename (lpszPath, lpszFile, NULL);
-          g_free (lpszPath);
-          g_free (lpszFile);
-          lpszFile = lpszTemp;
+          else
+            {
+              s += i - j;
+            }
+          if (G_IS_DIR_SEPARATOR (szFile[i]))
+            szLongName[s++] = G_DIR_SEPARATOR;
+          szLongName[s] = '\0';
+          j = i + 1;
         }
-      dwSize = GetFullPathNameA (lpszFile, MAX_PATH, szFile, &p);
-      g_free (lpszFile);
-      if (dwSize == 0 || dwSize >= MAX_PATH)
-        return g_strdup (file);
-      if (dwSize >= 2)
-        {
-          if (szFile[1] == ':')
-            {
-              szLongName[0] = (CHAR)CharUpperA ((LPSTR)szFile[0]);
-              szLongName[1] = ':';
-              if (G_IS_DIR_SEPARATOR (szFile[2]))
-                {
-                  szLongName[2] = G_DIR_SEPARATOR;
-                  i = 3;
-                }
-              else
-                {
-                  i = 2;
-                }
-            }
-          else if (G_IS_DIR_SEPARATOR (szFile[0])
-                                            && G_IS_DIR_SEPARATOR (szFile[1]))
-            {
-              i = 2;
-              while (i < dwSize)
-                {
-                  if (G_IS_DIR_SEPARATOR (szFile[i]))
-                    break;
-                  i += IsDBCSLeadByteEx (CP_ACP, szFile[i]) ? 2 : 1;
-                }
-              i++;
-              g_memmove (szLongName, szFile, i * sizeof (CHAR));
-            }
-        }
-      szLongName[i] = '\0';
-      j = i;
-      while (i <= dwSize)
-        {
-          if (szFile[i] == '\0' || G_IS_DIR_SEPARATOR (szFile[i]))
-            {
-              WIN32_FIND_DATAA rcf;
-
-              s = lstrlenA (szLongName);
-              g_memmove (szLongName + s, szFile + j, (i - j) * sizeof (CHAR));
-              szLongName[s + i - j] = '\0';
-              hFind = FindFirstFileA (szLongName, &rcf);
-              if (hFind != INVALID_HANDLE_VALUE)
-                FindClose (hFind);
-              else
-                lstrcpyA (rcf.cFileName, ".");
-              if (lstrcmpA (rcf.cFileName, ".") != 0)
-                {
-                  g_memmove (szLongName + s, rcf.cFileName,
-                                    lstrlenA (rcf.cFileName) * sizeof (CHAR));
-                  s += lstrlenA (rcf.cFileName);
-                }
-              else
-                {
-                  s += i - j;
-                }
-              if (G_IS_DIR_SEPARATOR (szFile[i]))
-                szLongName[s++] = G_DIR_SEPARATOR;
-              szLongName[s] = '\0';
-              j = i + 1;
-            }
-          i += IsDBCSLeadByteEx (CP_ACP, szFile[i]) ? 2 : 1;
-        }
-      utf8str = g_locale_to_utf8 (szLongName, -1, NULL, NULL, NULL);
-# endif /* GLIB_CHECK_VERSION(2,6,0) || ! defined (UNICODE) */
-# if GLIB_CHECK_VERSION(2,6,0)
+# ifdef UNICODE
+      i++;
+# else /* not UNICODE */
+      i += IsDBCSLeadByteEx (CP_ACP, szFile[i]) ? 2 : 1;
+# endif /* not UNICODE */
     }
-# endif /* GLIB_CHECK_VERSION(2,6,0) */
+# ifdef UNICODE
+  utf8str = g_utf16_to_utf8 (szLongName, -1, NULL, NULL, NULL);
+# else /* not UNICODE */
+  utf8str = g_locale_to_utf8 (szLongName, -1, NULL, NULL, NULL);
+# endif /* not UNICODE */
   if (!utf8str)
     return g_strdup (file);
   filename = g_filename_from_utf8 (utf8str, -1, NULL, NULL, NULL);
@@ -484,12 +391,10 @@ fileio_open (const gchar *file,
   FileIO *fio;
 #ifdef G_OS_WIN32
   HANDLE hFile;
-# if ! GLIB_CHECK_VERSION(2,6,0)
   LPTSTR lpszFile;
-#  ifdef UNICODE
+# ifdef UNICODE
   gchar *utf8str;
-#  endif /* UNICODE */
-# endif /* not GLIB_CHECK_VERSION(2,6,0) */
+# endif /* UNICODE */
 #else /* not G_OS_WIN32 */
   int fd, flags;
   GList *glist;
@@ -515,41 +420,20 @@ fileio_open (const gchar *file,
       g_free (dir);
     }
 #ifdef G_OS_WIN32
-# if GLIB_CHECK_VERSION(2,6,0)
-  if (G_WIN32_HAVE_WIDECHAR_API ())
+# ifdef UNICODE
+  utf8str = g_filename_to_utf8 (file, -1, NULL, NULL, NULL);
+  if (!utf8str)
     {
-      gchar *utf8str;
-      LPWSTR lpszFile;
-# endif /* GLIB_CHECK_VERSION(2,6,0) */
-
-# if GLIB_CHECK_VERSION(2,6,0) || defined (UNICODE)
-      utf8str = g_filename_to_utf8 (file, -1, NULL, NULL, NULL);
-      if (!utf8str)
-        {
-          g_free (path);
-          return NULL;
-        }
-      lpszFile = g_utf8_to_utf16 (utf8str, -1, NULL, NULL, NULL);
-      hFile = CreateFileW (lpszFile, access, share, NULL, mode,
-                                                FILE_ATTRIBUTE_NORMAL, NULL);
-      g_free (lpszFile);
-# endif /* GLIB_CHECK_VERSION(2,6,0) || defined (UNICODE) */
-# if GLIB_CHECK_VERSION(2,6,0)
+      g_free (path);
+      return NULL;
     }
-  else
-    {
-      LPSTR lpszFile;
-# endif /* GLIB_CHECK_VERSION(2,6,0) */
-
-# if GLIB_CHECK_VERSION(2,6,0) || ! defined (UNICODE)
-      lpszFile = g_win32_locale_filename_from_utf8 (file);
-      hFile = CreateFileA (lpszFile, access, share, NULL, mode,
+  lpszFile = g_utf8_to_utf16 (utf8str, -1, NULL, NULL, NULL);
+# else /* not UNICODE */
+  lpszFile = g_win32_locale_filename_from_utf8 (file);
+# endif /* not UNICODE */
+  hFile = CreateFile (lpszFile, access, share, NULL, mode,
                                                 FILE_ATTRIBUTE_NORMAL, NULL);
-      g_free (lpszFile);
-# endif /* GLIB_CHECK_VERSION(2,6,0) || ! defined (UNICODE) */
-# if GLIB_CHECK_VERSION(2,6,0)
-    }
-# endif /* GLIB_CHECK_VERSION(2,6,0) */
+  g_free (lpszFile);
   if (hFile == INVALID_HANDLE_VALUE)
     {
       g_free (path);
@@ -1059,9 +943,9 @@ fileio_mkdir (const gchar *dir)
 {
   gboolean result;
   gchar *path, *parent, *utf8str;
-#if ! GLIB_CHECK_VERSION(2,6,0) && defined (G_OS_WIN32)
+#ifdef G_OS_WIN32
   LPTSTR lpszPath;
-#endif /* ! GLIB_CHECK_VERSION(2,6,0) && defined (G_OS_WIN32) */
+#endif /* G_OS_WIN32 */
 
   path = fileio_get_full_path (dir);
   parent = g_path_get_dirname (path);
@@ -1086,42 +970,23 @@ fileio_mkdir (const gchar *dir)
       g_free (parent);
       return TRUE;
     }
-  result = !g_file_test (parent, G_FILE_TEST_EXISTS) && !fileio_mkdir (parent);
+  result = g_file_test (parent, G_FILE_TEST_EXISTS) || fileio_mkdir (parent);
   g_free (parent);
-  if (result)
+  if (!result)
     {
       g_free (path);
       return FALSE;
     }
 #ifdef G_OS_WIN32
-# if GLIB_CHECK_VERSION(2,6,0)
-  if (G_WIN32_HAVE_WIDECHAR_API ())
-    {
-      LPWSTR lpszPath;
-# endif /* GLIB_CHECK_VERSION(2,6,0) */
-
-# if GLIB_CHECK_VERSION(2,6,0) || defined (UNICODE)
-      utf8str = g_filename_to_utf8 (path, -1, NULL, NULL, NULL);
-      lpszPath = g_utf8_to_utf16 (utf8str, -1, NULL, NULL, NULL);
-      g_free (utf8str);
-      result = CreateDirectoryW (lpszPath, NULL);
-      g_free (lpszPath);
-# endif /* GLIB_CHECK_VERSION(2,6,0) || defined (UNICODE) */
-# if GLIB_CHECK_VERSION(2,6,0)
-    }
-  else
-    {
-      LPSTR lpszPath;
-# endif /* GLIB_CHECK_VERSION(2,6,0) */
-
-# if GLIB_CHECK_VERSION(2,6,0) || ! defined (UNICODE)
-      lpszPath = g_win32_locale_filename_from_utf8 (path);
-      result = CreateDirectoryA (lpszPath, NULL);
-      g_free (lpszPath);
-# endif /* GLIB_CHECK_VERSION(2,6,0) || ! defined (UNICODE) */
-# if GLIB_CHECK_VERSION(2,6,0)
-    }
-# endif /* GLIB_CHECK_VERSION(2,6,0) */
+# ifdef UNICODE
+  utf8str = g_filename_to_utf8 (path, -1, NULL, NULL, NULL);
+  lpszPath = g_utf8_to_utf16 (utf8str, -1, NULL, NULL, NULL);
+  g_free (utf8str);
+# else /* not UNICODE */
+  lpszPath = g_win32_locale_filename_from_utf8 (path);
+# endif /* not UNICODE */
+  result = CreateDirectory (lpszPath, NULL);
+  g_free (lpszPath);
 #else /* not G_OS_WIN32 */
   result = g_mkdir (path, S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IXGRP
                                                     | S_IROTH | S_IXOTH) == 0;
@@ -1148,98 +1013,21 @@ fileio_remove (const gchar *path)
       if (!dir)
         return FALSE;
       while (result && (name = g_dir_read_name (dir)))
-        {
-          gchar *file;
+        if (g_strcmp (name, ".") != 0 && g_strcmp (name, "..") != 0)
+          {
+            gchar *file;
 
-          file = g_build_filename (path, name, NULL);
-          result = fileio_remove (file);
-          g_free (file);
-        }
+            file = g_build_filename (path, name, NULL);
+            result = fileio_remove (file);
+            g_free (file);
+          }
       g_dir_close (dir);
       if (result)
-        {
-#ifdef G_OS_WIN32
-# if ! GLIB_CHECK_VERSION(2,6,0)
-#  ifdef UNICODE
-          gchar *utf8str;
-#  endif /* UNICODE */
-          LPTSTR lpszPath;
-# endif /* not GLIB_CHECK_VERSION(2,6,0) */
-# if GLIB_CHECK_VERSION(2,6,0)
-          if (G_WIN32_HAVE_WIDECHAR_API ())
-            {
-              gchar *utf8str;
-              LPWSTR lpszPath;
-# endif /* GLIB_CHECK_VERSION(2,6,0) */
-
-# if GLIB_CHECK_VERSION(2,6,0) || defined (UNICODE)
-              utf8str = g_filename_to_utf8 (path, -1, NULL, NULL, NULL);
-              lpszPath = g_utf8_to_utf16 (utf8str, -1, NULL, NULL, NULL);
-              g_free (utf8str);
-              result = RemoveDirectoryW (lpszPath);
-              g_free (lpszPath);
-# endif /* GLIB_CHECK_VERSION(2,6,0) || defined (UNICODE) */
-# if GLIB_CHECK_VERSION(2,6,0)
-            }
-          else
-            {
-              LPSTR lpszPath;
-# endif /* GLIB_CHECK_VERSION(2,6,0) */
-
-# if GLIB_CHECK_VERSION(2,6,0) || ! defined (UNICODE)
-              lpszPath = g_win32_locale_filename_from_utf8 (path);
-              result = RemoveDirectoryA (lpszPath);
-              g_free (lpszPath);
-# endif /* GLIB_CHECK_VERSION(2,6,0) || ! defined (UNICODE) */
-# if GLIB_CHECK_VERSION(2,6,0)
-            }
-# endif /* GLIB_CHECK_VERSION(2,6,0) */
-#else /* not G_OS_WIN32 */
-          result = g_rmdir (path) == 0;
-#endif /* not G_OS_WIN32 */
-        }
+        result = g_rmdir (path) == 0;
     }
   else
     {
-#ifdef G_OS_WIN32
-# if ! GLIB_CHECK_VERSION(2,6,0)
-#  ifdef UNICODE
-      gchar *utf8str;
-#  endif /* UNICODE */
-      LPTSTR lpszPath;
-# endif /* not GLIB_CHECK_VERSION(2,6,0) */
-# if GLIB_CHECK_VERSION(2,6,0)
-      if (G_WIN32_HAVE_WIDECHAR_API ())
-        {
-          gchar *utf8str;
-          LPWSTR lpszPath;
-# endif /* GLIB_CHECK_VERSION(2,6,0) */
-
-# if GLIB_CHECK_VERSION(2,6,0) || defined (UNICODE)
-          utf8str = g_filename_to_utf8 (path, -1, NULL, NULL, NULL);
-          lpszPath = g_utf8_to_utf16 (utf8str, -1, NULL, NULL, NULL);
-          g_free (utf8str);
-          result = DeleteFileW (lpszPath);
-          g_free (lpszPath);
-# endif /* GLIB_CHECK_VERSION(2,6,0) || defined (UNICODE) */
-# if GLIB_CHECK_VERSION(2,6,0)
-        }
-      else
-        {
-          LPSTR lpszPath;
-# endif /* GLIB_CHECK_VERSION(2,6,0) */
-
-# if GLIB_CHECK_VERSION(2,6,0) || ! defined (UNICODE)
-          lpszPath = g_win32_locale_filename_from_utf8 (path);
-          result = DeleteFileA (lpszPath);
-          g_free (lpszPath);
-# endif /* GLIB_CHECK_VERSION(2,6,0) || ! defined (UNICODE) */
-# if GLIB_CHECK_VERSION(2,6,0)
-        }
-# endif /* GLIB_CHECK_VERSION(2,6,0) */
-#else /* not G_OS_WIN32 */
       result = g_remove (path) == 0;
-#endif /* not G_OS_WIN32 */
     }
   return result;
 }
@@ -1262,69 +1050,37 @@ fileio_gettime (const gchar *file,
 {
 #ifdef G_OS_WIN32
   HANDLE hFind;
-# if ! GLIB_CHECK_VERSION(2,6,0)
   LPTSTR lpszFile;
   WIN32_FIND_DATA rcFind;
-#  ifdef UNICODE
+# ifdef UNICODE
   gchar *utf8str;
-#  endif /* UNICODE */
-# endif /* not GLIB_CHECK_VERSION(2,6,0) */
+# endif /* UNICODE */
 #else /* not G_OS_WIN32 */
-  struct stat buf;
+  GStatBuf buf;
 #endif /* not G_OS_WIN32 */
 
   if (!file)
     return FALSE;
 #ifdef G_OS_WIN32
-# if GLIB_CHECK_VERSION(2,6,0)
-  if (G_WIN32_HAVE_WIDECHAR_API ())
-    {
-      gchar *utf8str;
-      LPWSTR lpszFile;
-      WIN32_FIND_DATAW rcFind;
-# endif /* GLIB_CHECK_VERSION(2,6,0) */
-
-# if GLIB_CHECK_VERSION(2,6,0) || defined (UNICODE)
-      utf8str = g_filename_to_utf8 (file, -1, NULL, NULL, NULL);
-      if (!utf8str)
-        return FALSE;
-      lpszFile = g_utf8_to_utf16 (utf8str, -1, NULL, NULL, NULL);
-      g_free (utf8str);
-      if (!lpszFile)
-        return FALSE;
-      hFind = FindFirstFileW (lpszFile, &rcFind);
-      g_free (lpszFile);
-      if (hFind == INVALID_HANDLE_VALUE || !FindClose (hFind))
-        return FALSE;
-      if (atime)
-        *atime = rcFind.ftLastAccessTime;
-      if (mtime)
-        *mtime = rcFind.ftLastWriteTime;
-# endif /* GLIB_CHECK_VERSION(2,6,0) || defined (UNICODE) */
-# if GLIB_CHECK_VERSION(2,6,0)
-    }
-  else
-    {
-      LPSTR lpszFile;
-      WIN32_FIND_DATAA rcFind;
-# endif /* GLIB_CHECK_VERSION(2,6,0) */
-
-# if GLIB_CHECK_VERSION(2,6,0) || !defined (UNICODE)
-      lpszFile = g_win32_locale_filename_from_utf8 (file);
-      if (!lpszFile)
-        return FALSE;
-      hFind = FindFirstFileA (lpszFile, &rcFind);
-      g_free (lpszFile);
-      if (hFind == INVALID_HANDLE_VALUE || !FindClose (hFind))
-        return FALSE;
-      if (atime)
-        *atime = rcFind.ftLastAccessTime;
-      if (mtime)
-        *mtime = rcFind.ftLastWriteTime;
-# endif /* GLIB_CHECK_VERSION(2,6,0) || !defined (UNICODE) */
-# if GLIB_CHECK_VERSION(2,6,0)
-    }
-# endif /* GLIB_CHECK_VERSION(2,6,0) */
+# ifdef UNICODE
+  utf8str = g_filename_to_utf8 (file, -1, NULL, NULL, NULL);
+  if (!utf8str)
+    return FALSE;
+  lpszFile = g_utf8_to_utf16 (utf8str, -1, NULL, NULL, NULL);
+  g_free (utf8str);
+# else /* not UNICODE */
+  lpszFile = g_win32_locale_filename_from_utf8 (file);
+# endif /* not UNICODE */
+  if (!lpszFile)
+    return FALSE;
+  hFind = FindFirstFile (lpszFile, &rcFind);
+  g_free (lpszFile);
+  if (hFind == INVALID_HANDLE_VALUE || !FindClose (hFind))
+    return FALSE;
+  if (atime)
+    *atime = rcFind.ftLastAccessTime;
+  if (mtime)
+    *mtime = rcFind.ftLastWriteTime;
 #else /* not G_OS_WIN32 */
   if (g_stat (file, &buf))
     return FALSE;
@@ -1347,76 +1103,50 @@ fileio_settime (const gchar *file,
                 FileIOTime  *atime,
                 FileIOTime  *mtime)
 {
-#ifdef G_OS_WIN32
-  HANDLE hFile;
-# if ! GLIB_CHECK_VERSION(2,6,0)
-  LPTSTR lpszFile;
-#  ifdef UNICODE
-  gchar *utf8str;
-#  endif /* UNICODE */
-# endif /* not GLIB_CHECK_VERSION(2,6,0) */
-#else /* not G_OS_WIN32 */
-  struct stat sbuf;
-  struct utimbuf ubuf;
-#endif /* not G_OS_WIN32 */
-
-  if (!file)
-    return FALSE;
-#ifdef G_OS_WIN32
-# if GLIB_CHECK_VERSION(2,6,0)
-  if (G_WIN32_HAVE_WIDECHAR_API ())
+#if defined (G_OS_WIN32) || defined (HAVE_UTIME)
+  if (file)
     {
+#endif /*defined (G_OS_WIN32) || defined (HAVE_UTIME) */
+#if defined (G_OS_WIN32)
+      HANDLE hFile;
+      LPTSTR lpszFile;
+# ifdef UNICODE
       gchar *utf8str;
-      LPWSTR lpszFile;
-# endif /* GLIB_CHECK_VERSION(2,6,0) */
 
-# if GLIB_CHECK_VERSION(2,6,0) || defined (UNICODE)
       utf8str = g_filename_to_utf8 (file, -1, NULL, NULL, NULL);
       if (!utf8str)
         return FALSE;
       lpszFile = g_utf8_to_utf16 (utf8str, -1, NULL, NULL, NULL);
       g_free (utf8str);
-      hFile = CreateFileW (lpszFile, GENERIC_WRITE,
-                                FILE_SHARE_READ | FILE_SHARE_WRITE, NULL,
-                                OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
-      g_free (lpszFile);
-# endif /* GLIB_CHECK_VERSION(2,6,0) || defined (UNICODE) */
-# if GLIB_CHECK_VERSION(2,6,0)
-    }
-  else
-    {
-      LPSTR lpszFile;
-# endif /* GLIB_CHECK_VERSION(2,6,0) */
-
-# if GLIB_CHECK_VERSION(2,6,0) || ! defined (UNICODE)
+# else /* not UNICODE */
       lpszFile = g_win32_locale_filename_from_utf8 (file);
-      hFile = CreateFileA (lpszFile, GENERIC_WRITE,
+# endif /* not UNICODE */
+      hFile = CreateFile (lpszFile, GENERIC_WRITE,
                                 FILE_SHARE_READ | FILE_SHARE_WRITE, NULL,
                                 OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
       g_free (lpszFile);
-# endif /* GLIB_CHECK_VERSION(2,6,0) || ! defined (UNICODE) */
-# if GLIB_CHECK_VERSION(2,6,0)
+      if (hFile == INVALID_HANDLE_VALUE)
+        return FALSE;
+      if (!SetFileTime (hFile, NULL, atime, mtime))
+        {
+          CloseHandle (hFile);
+          return FALSE;
+        }
+      return CloseHandle (hFile);
+#elif defined (HAVE_UTIME)
+      GStatBuf sbuf;
+      struct utimbuf ubuf;
+
+      if (g_stat (file, &sbuf) != 0)
+        return FALSE;
+      ubuf.actime  = atime ? *atime : sbuf.st_atime;
+      ubuf.modtime = mtime ? *mtime : sbuf.st_mtime;
+      return g_utime (file, &ubuf) == 0;
+#endif /* not HAVE_UTIME */
+#if defined (G_OS_WIN32) || defined (HAVE_UTIME)
     }
-# endif /* GLIB_CHECK_VERSION(2,6,0) */
-  if (hFile == INVALID_HANDLE_VALUE)
-    return FALSE;
-  if (!SetFileTime (hFile, NULL, atime, mtime))
-    {
-      CloseHandle (hFile);
-      return FALSE;
-    }
-  return CloseHandle (hFile);
-#else /* not G_OS_WIN32 */
-# ifdef HAVE_UTIME
-  if (g_stat (file, &sbuf))
-    return FALSE;
-  ubuf.actime  = atime ? *atime : sbuf.st_atime;
-  ubuf.modtime = mtime ? *mtime : sbuf.st_mtime;
-  return !utime (file, &ubuf);
-# else /* not HAVE_UTIME */
+#endif /*defined (G_OS_WIN32) || defined (HAVE_UTIME) */
   return FALSE;
-# endif /* not HAVE_UTIME */
-#endif /* not G_OS_WIN32 */
 }
 
 
@@ -1503,20 +1233,18 @@ GdkPixbuf *
 fileio_extract_icon (const gchar *file,
                      GtkIconSize  icon_size)
 {
+  GdkPixbuf *pixbuf = NULL;
 #if defined (G_OS_WIN32) || GLIB_CHECK_VERSION(2,16,0) || GLIB_CHECK_VERSION(2,4,0) && defined (MIMEFILE)
   gint icon_x, icon_y;
-  GdkPixbuf *pixbuf = NULL;
 # ifdef G_OS_WIN32
   HICON hIcon = NULL;
   ICONINFO ii;
   UINT uFlags;
-#  if ! GLIB_CHECK_VERSION(2,6,0)
   LPTSTR lpszFile;
   SHFILEINFO shfi;
-#   ifdef UNICODE
+#  ifdef UNICODE
   gchar *utf8str;
-#   endif /* UNICODE */
-#  endif /* not GLIB_CHECK_VERSION(2,6,0) */
+#  endif /* UNICODE */
 # endif /* G_OS_WIN32 */
 
   if (!gtk_icon_size_lookup (icon_size, &icon_x, &icon_y))
@@ -1531,51 +1259,24 @@ fileio_extract_icon (const gchar *file,
         default:                          icon_x =  0; icon_y =  0;
       }
   if (!file || icon_x <= 0 || icon_y <= 0)
-#endif /* defined (G_OS_WIN32) || GLIB_CHECK_VERSION(2,16,0) || GLIB_CHECK_VERSION(2,4,0) && defined (MIMEFILE) */
     return NULL;
-#if defined (G_OS_WIN32)
+# ifdef G_OS_WIN32
   uFlags = ABS (GetSystemMetrics (SM_CXSMICON) - icon_x)
                                 + ABS (GetSystemMetrics (SM_CYSMICON) - icon_y)
                                 < ABS (GetSystemMetrics (SM_CXICON) - icon_x)
                                 + ABS (GetSystemMetrics (SM_CYSMICON) - icon_y)
                                         ? SHGFI_SMALLICON : SHGFI_LARGEICON;
-# if GLIB_CHECK_VERSION(2,6,0)
-  if (G_WIN32_HAVE_WIDECHAR_API ())
-    {
-      gchar *utf8str;
-      LPWSTR lpszFile;
-      SHFILEINFOW shfi;
-# endif /* GLIB_CHECK_VERSION(2,6,0) */
-
-# if GLIB_CHECK_VERSION(2,6,0) || defined (UNICODE)
-      utf8str = g_filename_to_utf8 (file, -1, NULL, NULL, NULL);
-      if (!utf8str)
-        return NULL;
-      lpszFile = g_utf8_to_utf16 (file, -1, NULL, NULL, NULL);
-      if ((BOOL)SHGetFileInfoW (lpszFile, 0, &shfi, sizeof (SHFILEINFOW),
-                                SHGFI_ICON | SHGFI_USEFILEATTRIBUTES | uFlags))
-        hIcon = shfi.hIcon;
-      g_free (lpszFile);
-# endif /* GLIB_CHECK_VERSION(2,6,0) || defined (UNICODE) */
-# if GLIB_CHECK_VERSION(2,6,0)
-    }
-  else
-    {
-      LPSTR lpszFile;
-      SHFILEINFOA shfi;
-# endif /* GLIB_CHECK_VERSION(2,6,0) */
-
-# if GLIB_CHECK_VERSION(2,6,0) || defined (UNICODE)
-      lpszFile = g_win32_locale_filename_from_utf8 (file);
-      if ((BOOL)SHGetFileInfoA (lpszFile, 0, &shfi, sizeof (SHFILEINFOW),
-                                SHGFI_ICON | SHGFI_USEFILEATTRIBUTES | uFlags))
-        hIcon = shfi.hIcon;
-      g_free (lpszFile);
-# endif /* GLIB_CHECK_VERSION(2,6,0) || defined (UNICODE) */
-# if GLIB_CHECK_VERSION(2,6,0)
-    }
-# endif /* GLIB_CHECK_VERSION(2,6,0) */
-  if (GetIconInfo (hIcon, &ii))
+#  ifdef UNICODE
+  utf8str = g_filename_to_utf8 (file, -1, NULL, NULL, NULL);
+  if (!utf8str)
+    return NULL;
+  lpszFile = g_utf8_to_utf16 (file, -1, NULL, NULL, NULL);
+#  else /* not UNICODE */
+  lpszFile = g_win32_locale_filename_from_utf8 (file);
+#  endif /* not UNICODE */
+  if ((BOOL)SHGetFileInfo (lpszFile, 0, &shfi, sizeof (SHFILEINFOW),
+                                SHGFI_ICON | SHGFI_USEFILEATTRIBUTES | uFlags)
+                                            && GetIconInfo (shfi.hIcon, &ii))
     {
       BITMAPINFOHEADER bmih;
       HDC hDC;
@@ -1638,10 +1339,10 @@ fileio_extract_icon (const gchar *file,
       DeleteDC (hDC);
       DeleteObject (ii.hbmColor);
       DeleteObject (ii.hbmMask);
+      DestroyIcon (shfi.hIcon);
     }
-  if (hIcon)
-    DestroyIcon (hIcon);
-#elif GLIB_CHECK_VERSION(2,16,0) || GLIB_CHECK_VERSION(2,4,0) && defined (MIMEFILE)
+  g_free (lpszFile);
+# else /* not G_OS_WIN32 */
   if (g_file_test (file, G_FILE_TEST_IS_DIR))
     {
       const gchar *home;
@@ -1661,7 +1362,7 @@ fileio_extract_icon (const gchar *file,
   else
     {
       gchar *mime_type = NULL;
-# if GLIB_CHECK_VERSION(2,16,0)
+#  if GLIB_CHECK_VERSION(2,16,0)
       gboolean uncertain;
       gchar *content_type;
 
@@ -1671,12 +1372,12 @@ fileio_extract_icon (const gchar *file,
           mime_type = g_content_type_get_mime_type (content_type);
           g_free (content_type);
         }
-# endif /* GLIB_CHECK_VERSION(2,16,0) */
-# ifdef MIMEFILE
-#  if GLIB_CHECK_VERSION(2,16,0)
+#  endif /* GLIB_CHECK_VERSION(2,16,0) */
+#  ifdef MIMEFILE
+#   if GLIB_CHECK_VERSION(2,16,0)
       if (!mime_type)
         {
-#  endif /* GLIB_CHECK_VERSION(2,16,0) */
+#   endif /* GLIB_CHECK_VERSION(2,16,0) */
           gchar *text;
           gssize length;
           text = fileio_load (MIMEFILE, &length);
@@ -1713,10 +1414,10 @@ fileio_extract_icon (const gchar *file,
               g_free (utf8str);
               g_free (text);
             }
-#  if GLIB_CHECK_VERSION(2,16,0)
+#   if GLIB_CHECK_VERSION(2,16,0)
         }
-#  endif /* GLIB_CHECK_VERSION(2,16,0) */
-# endif /* MIMEFILE */
+#   endif /* GLIB_CHECK_VERSION(2,16,0) */
+#  endif /* MIMEFILE */
       if (mime_type)
         {
           gchar *icon_name, *separator;
@@ -1742,7 +1443,7 @@ fileio_extract_icon (const gchar *file,
                     }
                   if (result)
                     {
-# if GLIB_CHECK_VERSION(2,6,0)
+#  if GLIB_CHECK_VERSION(2,6,0)
                       gint *sizes;
 
                       sizes = gtk_icon_theme_get_icon_sizes (icon_theme,
@@ -1762,7 +1463,7 @@ fileio_extract_icon (const gchar *file,
                                                         icon_name, n, 0, NULL);
                         }
                       if (!pixbuf)
-# endif /* GLIB_CHECK_VERSION(2,6,0) */
+#  endif /* GLIB_CHECK_VERSION(2,6,0) */
                         pixbuf = gtk_icon_theme_load_icon (icon_theme,
                                     icon_name, (icon_x + icon_y) / 2, 0, NULL);
                     }
@@ -1771,9 +1472,8 @@ fileio_extract_icon (const gchar *file,
           g_free (icon_name);
         }
     }
-#endif /* GLIB_CHECK_VERSION(2,16,0) || GLIB_CHECK_VERSION(2,4,0) && defined (MIMEFILE) */
-#if defined (G_OS_WIN32) || GLIB_CHECK_VERSION(2,16,0) || GLIB_CHECK_VERSION(2,4,0) && defined (MIMEFILE)
-  return pixbuf;
+# endif /* not G_OS_WIN32 */
 #endif /* defined (G_OS_WIN32) || GLIB_CHECK_VERSION(2,16,0) || GLIB_CHECK_VERSION(2,4,0) && defined (MIMEFILE) */
+  return pixbuf;
 }
 #endif /* not USE_GTK_EMULATE */
