@@ -1,10 +1,10 @@
 /*
     misc
-    copyright (c) 1998-2006 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2012 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
 
-    This program is free software; you can redistribute it and/or modify
+    This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -13,19 +13,16 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 #ifndef __MISC_H__
 #define __MISC_H__
 
 
-#include "common.h"
+#include "gcommon.h"
 
 
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
+G_BEGIN_DECLS
 
 
 #define MISC_CREATE_MENU_BAR       1
@@ -41,6 +38,7 @@ extern "C" {
 #define MISC_CREATE_MENU_DISABLE   32
 #define MISC_CREATE_MENU_MASK      7
 
+
 typedef struct _MiscCreateMenuEntry
 {
   gchar *path;
@@ -124,6 +122,23 @@ const gchar *
 misc_spec_locale (const gchar *prgname);
 
 
+/******************************************************************************
+*                                                                             *
+* ja:クローズボタン関数群                                                     *
+*                                                                             *
+******************************************************************************/
+/*  ja:クローズボタンのイメージを取得する
+    RET,イメージ,NULL:エラー                                                */
+GtkWidget *
+misc_close_image (void);
+
+
+/*  ja:クローズボタンを取得する
+    RET,ボタン,NULL:エラー                                                  */
+GtkWidget *
+misc_close_button (void);
+
+
 /******************************************************************************
 *                                                                             *
 * ja:低レベル関数群                                                           *
@@ -145,13 +160,13 @@ misc_mnemonic_to_text (const gchar *mnemonic);
          page,ページ
           pos,位置                                                          */
 void
-misc_set_scroll_bar (GtkWidget     *scroll,
-                     GtkSignalFunc  func,
-                     gpointer       func_data,
-                     const gint     min,
-                     const gint     max,
-                     const gint     page,
-                     const gint     pos);
+misc_set_scroll_bar (GtkWidget  *scroll,
+                     GCallback   func,
+                     gpointer    func_data,
+                     const gint  min,
+                     const gint  max,
+                     const gint  page,
+                     const gint  pos);
 
 
 /*  ja:Windowの内容をスクロールする
@@ -187,7 +202,7 @@ misc_dialog_key_press (GtkWidget   *widget,
 /*  ja:数値→文字列
     value,数値
     radix,基数
-     wide,桁数
+     wide,桁数(正:空白,負:0)
      flag,TRUE:符号あり,FALSE:符号なし
       RET,文字列                                                            */
 gchar *
@@ -235,7 +250,7 @@ misc_str_to_float (gint        *value,
      size,要素数
      bits,ビット数
     radix,基数
-     wide,桁数
+     wide,桁数(正:空白,負:0)
      flag,TRUE:符号あり,FALSE:符号なし
       RET,文字列                                                            */
 gchar *
@@ -262,9 +277,84 @@ misc_str_to_array (gsize          *size,
                    const gboolean  flag);
 
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+/******************************************************************************
+*                                                                             *
+* ja:日時文字列関数群                                                         *
+*                                                                             *
+******************************************************************************/
+/*  ja:日時文字列→日時
+    cymdhms,[[CC]YY]MMDDhhmm[.ss]文字列,接尾辞YMwDhmsの数値
+        RET,日時                                                            */
+GDateTime *
+misc_string_to_time (const gchar *cymdhms);
+
+
+/*  ja:日時文字列→日時
+    datetime,基準となる日時(NULL:現在時刻)
+     cymdhms,[[CC]YY]MMDDhhmm[.ss]文字列,接尾辞YMwDhmsの数値
+         RET,日時                                                           */
+GDateTime *
+misc_string_to_time_datetime (GDateTime   *datetime,
+                              const gchar *cymdhms);
+
+
+/*  ja:日時文字列→日時
+    datetime,基準となる日時(NULL:現在時刻)
+     cymdhms,[[CC]YY]MMDDhhmm[.ss]文字列,接尾辞YMwDhmsの数値
+        sign,デフォルトの符号,NULL:+
+         RET,日時                                                           */
+GDateTime *
+misc_string_to_time_full (GDateTime   *datetime,
+                          const gchar *cymdhms,
+                          const gchar  sign);
+
+
+/*  ja:日時文字列→日時
+    cymdhms,[[CC]YY]MMDDhhmm[.ss]文字列,接尾辞YMwDhmsの数値
+       sign,デフォルトの符号,NULL:+
+        RET,日時                                                            */
+GDateTime *
+misc_string_to_time_sign (const gchar *cymdhms,
+                          const gchar  sign);
+
+
+/*  ja:日時文字列→UNIX時間
+    cymdhms,[[CC]YY]MMDDhhmm[.ss]文字列,接尾辞YMwDhmsの数値
+        RET,日時                                                            */
+gint64
+misc_string_to_epoch (const gchar *cymdhms);
+
+
+/*  ja:日時文字列→UNIX時間
+    datetime,基準となる日時(NULL:現在時刻)
+     cymdhms,[[CC]YY]MMDDhhmm[.ss]文字列,接尾辞YMwDhmsの数値
+         RET,日時                                                           */
+gint64
+misc_string_to_epoch_datetime (GDateTime   *datetime,
+                               const gchar *cymdhms);
+
+
+/*  ja:日時文字列→UNIX時間
+    datetime,基準となる日時(NULL:現在時刻)
+     cymdhms,[[CC]YY]MMDDhhmm[.ss]文字列,接尾辞YMwDhmsの数値
+        sign,デフォルトの符号,NULL:+
+         RET,日時                                                           */
+gint64
+misc_string_to_epoch_full (GDateTime   *datetime,
+                           const gchar *cymdhms,
+                           const gchar  sign);
+
+
+/*  ja:日時文字列→UNIX時間
+    cymdhms,[[CC]YY]MMDDhhmm[.ss]文字列,接尾辞YMwDhmsの数値
+       sign,デフォルトの符号,NULL:+
+        RET,日時                                                            */
+gint64
+misc_string_to_epoch_sign (const gchar *cymdhms,
+                           const gchar  sign);
+
+
+G_END_DECLS
 
 
 #endif /* __MISC_H__ */
