@@ -1,6 +1,6 @@
 /*
     orzcomm
-    copyright (c) 1998-2008 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2012 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -152,18 +152,17 @@ DdemlCallback (UINT     uType,
                DWORD    dwData1,
                DWORD    dwData2)
 {
-  guint i, length;
+  GList *gl;
 
 # ifdef USE_THREAD
   G_LOCK (critical);
   critical = TRUE;
 # endif /* USE_THREAD */
-  length = g_list_length (glist);
-  for (i = 0; i < length; i++)
+  for (gl = g_list_first (glist); gl; gl = g_list_next (gl))
     {
       OrzComm *comm;
 
-      comm = g_list_nth_data (glist, i);
+      comm = gl->data;
       if (!comm)
         break;
       switch (uType)
@@ -215,10 +214,7 @@ orz_comm_selection_get (GtkWidget        *widget,
     {
       /* ja:他プロセスがプロセスの存在を確認しにきたとき
             このプロセスの存在を知らせる */
-      gchar *text;
-
-      text = "Orz Comm Selection";
-      gtk_selection_data_set_text (data, text, g_strlen (text));
+      gtk_selection_data_set_text (data, "Orz Comm Selection", -1);
     }
   else if (data->selection == comm->atom1)
     {
@@ -230,7 +226,7 @@ orz_comm_selection_get (GtkWidget        *widget,
   else if (data->selection == comm->atom2 && comm->arg)
     {
       /* ja:他プロセスが引数を取りにきたのでこのプロセスの引数を渡す */
-      gtk_selection_data_set_text (data, comm->arg, g_strlen (comm->arg));
+      gtk_selection_data_set_text (data, comm->arg, -1);
       g_free (comm->arg);
       comm->arg = NULL;
     }
@@ -258,10 +254,14 @@ orz_comm_selection_received (GtkWidget        *widget,
     }
   else if (data->selection == comm->atom2 && data->length > 0)
     {
-      gchar **argv;
+      gchar *utf8str, *compressed, **argv;
 
       /* ja:他にプロセスのコマンドラインを受け取ったとき */
-      argv = g_strsplit ((const gchar *)data->data, "\n", 0);
+      utf8str = (gchar *)gtk_selection_data_get_text (data);
+      compressed = g_strcompress (utf8str);
+      g_free (utf8str);
+      argv = g_strsplit (compressed, "\n", 0);
+      g_free (compressed);
       g_signal_emit_by_name (G_OBJECT (comm), "recv", argv);
       g_strfreev (argv);
     }
@@ -433,7 +433,7 @@ orz_comm_send (OrzComm     *comm,
                const gint   argc,
                gchar      **argv)
 {
-  return orz_comm_send_with_files (comm, argc, argv, NULL);
+  orz_comm_send_with_files (comm, argc, argv, NULL);
 }
 
 
@@ -467,16 +467,16 @@ orz_comm_send_with_files (OrzComm         *comm,
           gchar *tmp;
 
           tmp = fileio_get_full_path (argv[i]);
-          leng = g_strlen (tmp) * sizeof (gchar);
-          arg = g_realloc (arg, length + leng + sizeof (gchar));
-          g_memmove (arg + length, tmp, leng);
+          leng = g_strlen (tmp);
+          arg = g_realloc (arg, (length + leng + 1) * sizeof (gchar));
+          g_memmove (arg + length, tmp, leng * sizeof (gchar));
           g_free (tmp);
         }
       else
         {
-          leng = g_strlen (argv[i]) * sizeof (gchar);
-          arg = g_realloc (arg, length + leng + sizeof (gchar));
-          g_memmove (arg + length, argv[i], leng);
+          leng = g_strlen (argv[i]);
+          arg = g_realloc (arg, (length + leng + 1) * sizeof (gchar));
+          g_memmove (arg + length, argv[i], leng * sizeof (gchar));
         }
       length += leng;
       arg[length++] = '\n';
@@ -508,7 +508,8 @@ orz_comm_send_with_files (OrzComm         *comm,
   if (length > 0)
     {
       arg[length - 1] = '\0';
-      comm->arg = arg;
+      comm->arg = g_strescape (arg, NULL);
+      g_free (arg);
       gtk_selection_owner_set (comm->window, comm->atom2, GDK_CURRENT_TIME);
       gtk_selection_convert (comm->window, comm->atom1,
                                 GDK_SELECTION_TYPE_STRING, GDK_CURRENT_TIME);
