@@ -1,6 +1,6 @@
 /*
     Melody maid
-    copyright (c) 1998-2009 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2011 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -190,67 +190,95 @@ add_dialog (MmaidWindow *mmaid)
     {
       id3tag = mm_list_get_nth (MM_LIST (mmaid->list), i);
       if (id3tag->title && !g_list_find_custom (glist_title, id3tag->title,
-                                                        (GCompareFunc)strcmp))
+                                                    (GCompareFunc)g_strcmp))
         glist_title = g_list_insert_sorted (glist_title,
-                            g_strdup (id3tag->title), (GCompareFunc)strcmp);
+                        g_strdup (id3tag->title), (GCompareFunc)g_strcmp);
       if (id3tag->artist && !g_list_find_custom (glist_artist, id3tag->artist,
-                                                        (GCompareFunc)strcmp))
+                                                    (GCompareFunc)g_strcmp))
         glist_artist = g_list_insert_sorted (glist_artist,
-                            g_strdup (id3tag->artist), (GCompareFunc)strcmp);
+                        g_strdup (id3tag->artist), (GCompareFunc)g_strcmp);
       if (id3tag->album && !g_list_find_custom (glist_album, id3tag->album,
-                                                        (GCompareFunc)strcmp))
+                                                    (GCompareFunc)g_strcmp))
         glist_album = g_list_insert_sorted (glist_album,
-                            g_strdup (id3tag->album), (GCompareFunc)strcmp);
+                        g_strdup (id3tag->album), (GCompareFunc)g_strcmp);
       if (id3tag->comment && !g_list_find_custom (glist_comment,
-                                        id3tag->comment, (GCompareFunc)strcmp))
+                                    id3tag->comment, (GCompareFunc)g_strcmp))
         glist_comment = g_list_insert_sorted (glist_comment,
-                            g_strdup (id3tag->comment), (GCompareFunc)strcmp);
+                        g_strdup (id3tag->comment), (GCompareFunc)g_strcmp);
     }
   for (i = 0; i < 256; i++)
     if (genre[0][i] && !g_list_find_custom (glist_genre, genre[0][i],
-                                                        (GCompareFunc)strcmp))
+                                                    (GCompareFunc)g_strcmp))
       glist_genre = g_list_append (glist_genre, g_strdup (genre[0][i]));
   /* ja:コンボボックス */
 #if GTK_CHECK_VERSION(2,4,0)
+# if GTK_CHECK_VERSION(2,24,0)
+  combo_title = gtk_combo_box_text_new_with_entry ();
+  combo_artist = gtk_combo_box_text_new_with_entry ();
+  combo_album = gtk_combo_box_text_new_with_entry ();
+  combo_comment = gtk_combo_box_text_new_with_entry ();
+  combo_genre = gtk_combo_box_text_new_with_entry ();
+# else /* not GTK_CHECK_VERSION(2,24,0) */
   combo_title = gtk_combo_box_entry_new_text ();
   combo_artist = gtk_combo_box_entry_new_text ();
   combo_album = gtk_combo_box_entry_new_text ();
   combo_comment = gtk_combo_box_entry_new_text ();
   combo_genre = gtk_combo_box_entry_new_text ();
-  for (glist_title = g_list_first (glist_title); glist_title;
-            glist_title = g_list_delete_link (glist_title, glist_title))
+# endif /* not GTK_CHECK_VERSION(2,24,0) */
+  while ((glist_title = g_list_first (glist_title)))
     {
+# if GTK_CHECK_VERSION(2,24,0)
+      gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo_title),
+# else /* not GTK_CHECK_VERSION(2,24,0) */
       gtk_combo_box_append_text (GTK_COMBO_BOX (combo_title),
+# endif /* not GTK_CHECK_VERSION(2,24,0) */
                                                         glist_title->data);
       g_free (glist_title->data);
+      glist_title = g_list_delete_link (glist_title, glist_title);
     }
-  for (glist_artist = g_list_first (glist_artist); glist_artist;
-            glist_artist = g_list_delete_link (glist_artist, glist_artist))
+  while ((glist_artist = g_list_first (glist_artist)))
     {
+# if GTK_CHECK_VERSION(2,24,0)
+      gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo_artist),
+# else /* not GTK_CHECK_VERSION(2,24,0) */
       gtk_combo_box_append_text (GTK_COMBO_BOX (combo_artist),
+# endif /* not GTK_CHECK_VERSION(2,24,0) */
                                                         glist_artist->data);
       g_free (glist_artist->data);
+      glist_artist = g_list_delete_link (glist_artist, glist_artist);
     }
-  for (glist_album = g_list_first (glist_album); glist_album;
-            glist_album = g_list_delete_link (glist_album, glist_album))
+  while ((glist_album = g_list_first (glist_album)))
     {
+# if GTK_CHECK_VERSION(2,24,0)
+      gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo_album),
+# else /* not GTK_CHECK_VERSION(2,24,0) */
       gtk_combo_box_append_text (GTK_COMBO_BOX (combo_album),
+# endif /* not GTK_CHECK_VERSION(2,24,0) */
                                                         glist_album->data);
       g_free (glist_album->data);
+      glist_album = g_list_delete_link (glist_album, glist_album);
     }
-  for (glist_comment = g_list_first (glist_comment); glist_comment;
-            glist_comment = g_list_delete_link (glist_comment, glist_comment))
+  while ((glist_comment = g_list_first (glist_comment)))
     {
+# if GTK_CHECK_VERSION(2,24,0)
+      gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo_comment),
+# else /* not GTK_CHECK_VERSION(2,24,0) */
       gtk_combo_box_append_text (GTK_COMBO_BOX (combo_comment),
+# endif /* not GTK_CHECK_VERSION(2,24,0) */
                                                         glist_comment->data);
       g_free (glist_comment->data);
+      glist_comment = g_list_delete_link (glist_comment, glist_comment);
     }
-  for (glist_genre = g_list_first (glist_genre); glist_genre;
-            glist_genre = g_list_delete_link (glist_genre, glist_genre))
+  while ((glist_genre = g_list_first (glist_genre)))
     {
+# if GTK_CHECK_VERSION(2,24,0)
+      gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo_genre),
+# else /* not GTK_CHECK_VERSION(2,24,0) */
       gtk_combo_box_append_text (GTK_COMBO_BOX (combo_genre),
+# endif /* not GTK_CHECK_VERSION(2,24,0) */
                                                         glist_genre->data);
       g_free (glist_genre->data);
+      glist_genre = g_list_delete_link (glist_genre, glist_genre);
     }
 #else /* not GTK_CHECK_VERSION(2,4,0) */
   combo_title = gtk_combo_new ();
@@ -498,7 +526,8 @@ add_dialog (MmaidWindow *mmaid)
   gtk_table_attach (GTK_TABLE (table), hbox, 2, 3, 6, 7,
                                                     GTK_FILL, GTK_FILL, 0, 0);
   gtk_table_attach_defaults (GTK_TABLE (table), adddlg.charset, 3, 4, 6, 7);
-  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (adddlg.dialog)->vbox), table);
+  gtk_container_add (GTK_CONTAINER
+            (gtk_dialog_get_content_area (GTK_DIALOG (adddlg.dialog))), table);
 
   /* ja:表示 */
   gtk_dialog_set_response_sensitive (GTK_DIALOG (adddlg.dialog),
