@@ -1,6 +1,6 @@
 /*
     Melody maid
-    copyright (c) 1998-2009 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2011 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -29,6 +29,7 @@
 #include "process.h"
 #include "reload.h"
 #include "root.h"
+#include "sigmain.h"
 #include "version.h"
 #include "misc/fileio.h"
 #include "misc/misc.h"
@@ -139,11 +140,7 @@ command_save (GtkWidget *widget,
       gchar *path;
       gint leng;
 
-#if GLIB_CHECK_VERSION(2,6,0)
       path = g_filename_display_name (root);
-#else /* not GLIB_CHECK_VERSION(2,6,0) */
-      path = g_filename_to_utf8 (root, -1, NULL, NULL, NULL);
-#endif /* not GLIB_CHECK_VERSION(2,6,0) */
       leng = g_strlen (path);
       for (i = mm_list_length (MM_LIST (mmaid->list)) - 1; i >= 0; i--)
         {
@@ -168,11 +165,7 @@ command_save (GtkWidget *widget,
           gchar *utf8str;
           GtkWidget *dialog;
 
-#if GLIB_CHECK_VERSION(2,6,0)
           utf8str = g_filename_display_name (file);
-#else /* not GLIB_CHECK_VERSION(2,6,0) */
-          utf8str = g_filename_to_utf8 (file, -1, NULL, NULL, NULL);
-#endif /* not GLIB_CHECK_VERSION(2,6,0) */
           dialog = gtk_message_dialog_new (GTK_WINDOW (window),
                     GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                     GTK_MESSAGE_INFO, GTK_BUTTONS_YES_NO,
@@ -220,7 +213,9 @@ command_save_changed (GtkWidget *widget,
   gchar *root, *utf8str;
 
 #if GTK_CHECK_VERSION(2,4,0)
-# if GTK_CHECK_VERSION(2,6,0)
+# if GTK_CHECK_VERSION(2,24,0)
+  utf8str = gtk_combo_box_text_get_active_text (GTK_COMBO_BOX_TEXT (widget));
+# elif GTK_CHECK_VERSION(2,6,0)
   utf8str = gtk_combo_box_get_active_text (GTK_COMBO_BOX (widget));
 # else /* not GTK_CHECK_VERSION(2,6,0) */
   GtkTreeIter iter;
@@ -398,7 +393,17 @@ command_saveas (GtkWidget *widget,
   g_signal_connect (G_OBJECT (radio1), "toggled",
                             G_CALLBACK (command_save_toggled_radio1), hbox0);
   /* ja:コンボボックス */
-#if GTK_CHECK_VERSION(2,4,0)
+#if GTK_CHECK_VERSION(2,24,0)
+  combo0 = gtk_combo_box_text_new ();
+  for (gl = g_list_first (glist); gl; gl = g_list_next (gl))
+    {
+      gtk_combo_box_text_prepend_text (GTK_COMBO_BOX_TEXT (combo0), gl->data);
+      if (g_strfilecmp (gl->data, root) == 0)
+        gtk_combo_box_set_active (GTK_COMBO_BOX (combo0), 0);
+    }
+  g_signal_connect (G_OBJECT (combo0), "changed",
+                                    G_CALLBACK (command_save_changed), dialog);
+#elif GTK_CHECK_VERSION(2,4,0)
   combo0 = gtk_combo_box_new_text ();
   for (gl = g_list_first (glist); gl; gl = g_list_next (gl))
     {
@@ -475,7 +480,9 @@ command_saveas (GtkWidget *widget,
 # endif /* not GTK_CHECK_VERSION(2,6,0) */
 
       file = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
-# if GTK_CHECK_VERSION(2,6,0)
+# if GTK_CHECK_VERSION(2,24,0)
+      root = gtk_combo_box_text_get_active_text (GTK_COMBO_BOX_TEXT (combo0));
+# elif GTK_CHECK_VERSION(2,6,0)
       root = gtk_combo_box_get_active_text (GTK_COMBO_BOX (combo0));
 # else /* not GTK_CHECK_VERSION(2,6,0) */
       if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (combo0), &iter))
@@ -535,11 +542,7 @@ command_saveas (GtkWidget *widget,
         {
           gchar *utf8str;
 
-#if GLIB_CHECK_VERSION(2,6,0)
           utf8str = g_filename_display_name (file);
-#else /* not GLIB_CHECK_VERSION(2,6,0) */
-          utf8str = g_filename_to_utf8 (file, -1, NULL, NULL, NULL);
-#endif /* not GLIB_CHECK_VERSION(2,6,0) */
           dialog = gtk_message_dialog_new (GTK_WINDOW (window),
                     GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                     GTK_MESSAGE_INFO, GTK_BUTTONS_YES_NO,
@@ -758,7 +761,7 @@ command_exit (GtkWidget *widget,
   GdkEvent event;
 
   event.any.type = GDK_DELETE;
-  event.any.window = window->window;
+  event.any.window = gtk_widget_get_window (window);
   event.any.send_event = FALSE;
   gdk_event_put (&event);
 }
@@ -1210,26 +1213,21 @@ void
 command_copy (GtkWidget *widget,
               gpointer   user_data)
 {
-#ifndef G_OS_WIN32
-  if (gtk_selection_owner_set (window, GDK_SELECTION_CLIPBOARD,
-                                                            GDK_CURRENT_TIME))
-    {
-#endif /* not G_OS_WIN32 */
-      gint i, count;
-      MmaidWindow *mmaid;
+  GtkTargetEntry targets[1] = {{"Melody maid ID3 Tag", 0, TARGET_MMAID}};
+  gchar *clipboard_id3tag = NULL;
+  gint i, count;
+  MmaidWindow *mmaid;
 
-      mmaid = orz_mdi_get_data (ORZ_MDI (mdi), -1);
-      g_free (clipboard_id3tag);
-      clipboard_id3tag = NULL;
-      count = mm_list_length (MM_LIST (mmaid->list));
-      for (i = 0; i < count; i++)
-        if (mm_list_is_selected (MM_LIST (mmaid->list), i))
-          {
-            gchar *str;
-            const ID3Tag *id3tag;
+  mmaid = orz_mdi_get_data (ORZ_MDI (mdi), -1);
+  count = mm_list_length (MM_LIST (mmaid->list));
+  for (i = 0; i < count; i++)
+    if (mm_list_is_selected (MM_LIST (mmaid->list), i))
+      {
+        gchar *str;
+        const ID3Tag *id3tag;
 
-            id3tag = mm_list_get_nth (MM_LIST (mmaid->list), i);
-            str = g_strdup_printf ("%s\t%s\t%s\t%s\t%s\t%s\t%d\t%s\t%s",
+        id3tag = mm_list_get_nth (MM_LIST (mmaid->list), i);
+        str = g_strdup_printf ("%s\t%s\t%s\t%s\t%s\t%s\t%d\t%s\t%s",
                                     id3tag->file    ? id3tag->file     : "",
                                     id3tag->title   ? id3tag->title    : "",
                                     id3tag->artist  ? id3tag->artist   : "",
@@ -1239,56 +1237,26 @@ command_copy (GtkWidget *widget,
                                     id3tag->track,
                                     id3tag->genre   ? id3tag->genre    : "",
                                     id3tag->charset ? id3tag->charset  : "");
-            if (clipboard_id3tag)
-              {
-                gchar *tmp;
+        if (clipboard_id3tag)
+          {
+            gchar *tmp;
 
-                tmp = g_strconcat (clipboard_id3tag, "\n", str, NULL);
-                g_free (clipboard_id3tag);
-                g_free (str);
-                clipboard_id3tag = tmp;
-              }
-            else
-              {
-                clipboard_id3tag = str;
-              }
+            tmp = g_strconcat (clipboard_id3tag, "\n", str, NULL);
+            g_free (clipboard_id3tag);
+            g_free (str);
+            clipboard_id3tag = tmp;
           }
-#ifdef G_OS_WIN32
-      if (clipboard_id3tag)
-        {
-          HGLOBAL hGlobal;
-          LPSTR lpID3Tag;
-
-          hGlobal = GlobalAlloc (GMEM_MOVEABLE,
-                        (g_strlen (clipboard_id3tag) + 1) * sizeof (gchar));
-          if (hGlobal)
-            {
-              lpID3Tag = GlobalLock (hGlobal);
-              if (lpID3Tag)
-                {
-                  g_strcpy (lpID3Tag, clipboard_id3tag);
-                  GlobalUnlock (hGlobal);
-                  if (OpenClipboard (GDK_WINDOW_HWND (window->window)))
-                    {
-                      if (!EmptyClipboard ()
-                                    || !SetClipboardData (uFormat, hGlobal))
-                        GlobalFree (hGlobal);
-                      CloseClipboard ();
-                    }
-                  else
-                    {
-                      GlobalFree (hGlobal);
-                    }
-                }
-              else
-                {
-                  GlobalFree (hGlobal);
-                }
-            }
-        }
-#else /* not G_OS_WIN32 */
-    }
-#endif /* not G_OS_WIN32 */
+        else
+          {
+            clipboard_id3tag = str;
+          }
+      }
+  gtk_clipboard_set_with_data (gtk_clipboard_get (GDK_SELECTION_CLIPBOARD),
+                               targets,
+                               G_N_ELEMENTS (targets),
+                               signal_clipboard_get,
+                               signal_clipboard_clear,
+                               clipboard_id3tag);
 }
 
 
@@ -1296,39 +1264,8 @@ void
 command_paste (GtkWidget *widget,
                gpointer   user_data)
 {
-  /* ja:Windowsのセレクションの実装があやしいので暫定的処理 */
-#ifdef G_OS_WIN32
-  gchar *id3tag = NULL;
-  HGLOBAL hGlobal;
-  LPSTR lpID3Tag;
-
-  if (OpenClipboard (GDK_WINDOW_HWND (window->window)))
-    {
-      if (hGlobal = GetClipboardData (uFormat))
-        {
-          if (lpID3Tag = GlobalLock (hGlobal))
-            id3tag = g_strdup (lpID3Tag);
-          GlobalUnlock (hGlobal);
-        }
-      CloseClipboard ();
-      if (id3tag)
-        {
-          GtkSelectionData data;
-
-          data.selection = GDK_SELECTION_CLIPBOARD;
-          data.target = atom_id3tag;
-          data.type = atom_id3tag;
-          data.format = 8;
-          data.data = id3tag;
-          data.length = g_strlen (id3tag);
-          signal_selection_received (window, &data, GDK_CURRENT_TIME, NULL);
-          g_free (id3tag);
-        }
-    }
-#else /* not G_OS_WIN32 */
-  gtk_selection_convert (window, GDK_SELECTION_CLIPBOARD, atom_targets,
-                                                            GDK_CURRENT_TIME);
-#endif /* not G_OS_WIN32 */
+  gtk_clipboard_request_contents (gtk_clipboard_get (GDK_SELECTION_CLIPBOARD),
+                                atom_id3tag, signal_clipboard_received, NULL);
 }
 
 
