@@ -1,6 +1,6 @@
 /*
     Melody maid
-    copyright (c) 1998-2011 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2012 Kazuki Iwamoto http://www.maid.org/ iwm@maid.org
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,6 +24,9 @@
 #include "misc/misc.h"
 #include "orz/orzhistory.h"
 #include "orz/orzmdi.h"
+#if GLIB_CHECK_VERSION(2,16,0)
+# include <gio/gio.h>
+#endif /* GLIB_CHECK_VERSION(2,16,0) */
 
 
 /******************************************************************************
@@ -89,40 +92,56 @@ signal_drag_data_received (GtkWidget        *widget,
 
       i = gtk_notebook_get_current_page (GTK_NOTEBOOK (mdi));
       mmaid = i >= 0 ? orz_mdi_get_data (ORZ_MDI (mdi), i) : NULL;
-      for (i = 0; files[i]; i++)
+      for (i = 0; files[i] && *files[i] !='\0'; i++)
         {
+#if GLIB_CHECK_VERSION(2,16,0)
+          GFile *gfile;
+
+          gfile = g_file_new_for_uri (files[i]);
+          if (gfile)
+            {
+              gchar *file;
+
+              file = g_file_get_path (gfile);
+              g_object_unref (gfile);
+#else /* not GLIB_CHECK_VERSION(2,16,0) */
           gchar *file;
 
           file = g_filename_from_uri (files[i], NULL, NULL);
-          if (file)
-            {
-              switch (file_is_type (file))
+#endif /* not GLIB_CHECK_VERSION(2,16,0) */
+              if (file)
                 {
-                  case FILE_KIND_ERROR:
-                  case FILE_KIND_DB:
-                  case FILE_KIND_LIST:
-                    mmaid = file_open_edit (file);
-                    break;
-                  case FILE_KIND_UNKNOWN:
-                  case FILE_KIND_TAG:
+                  switch (file_is_type (file))
                     {
-                      ID3Tag *id3tag;
-
-                      id3tag = file_open_id3tag (file, charset_list);
-                      if (id3tag)
+                      case FILE_KIND_ERROR:
+                      case FILE_KIND_DB:
+                      case FILE_KIND_LIST:
+                        mmaid = file_open_edit (file);
+                        break;
+                      case FILE_KIND_UNKNOWN:
+                      case FILE_KIND_TAG:
                         {
-                          if (!mmaid || mm_list_search (MM_LIST (mmaid->list),
+                          ID3Tag *id3tag;
+
+                          id3tag = file_open_id3tag (file, charset_list);
+                          if (id3tag)
+                            {
+                              if (!mmaid
+                                    || mm_list_search (MM_LIST (mmaid->list),
                                         MM_LIST_FILE, id3tag->file, -1) >= 0)
-                            mmaid = file_open_edit (NULL);
-                          mm_list_append (MM_LIST (mmaid->list), id3tag);
-                          orz_mdi_set_edited (ORZ_MDI (mdi),
-                            orz_mdi_get_page_from_data (ORZ_MDI (mdi), mmaid),
-                                                                        TRUE);
+                                mmaid = file_open_edit (NULL);
+                              mm_list_append (MM_LIST (mmaid->list), id3tag);
+                              orz_mdi_set_edited (ORZ_MDI (mdi),
+                                orz_mdi_get_page_from_data (ORZ_MDI (mdi),
+                                                                mmaid), TRUE);
+                            }
                         }
                     }
+                  g_free (file);
                 }
-              g_free (file);
+#if GLIB_CHECK_VERSION(2,16,0)
             }
+#endif /* GLIB_CHECK_VERSION(2,16,0) */
          }
       g_strfreev (files);
     }
